clear
clear matrix
clear mata
capture log close
set more off
set varabbrev off


* Input: model_location.dta
* Output: model2INST

************************************************************************************
*THIS FILE CREATES ALL VARIABLES USED FOR ESTIMATION
************************************************************************************
use model_location.dta

* Keep only large markets
keep if ma==2|ma==3|ma==6
decode ma, gen(mastring)
drop ma

encode mastring, gen(manew)
rename manew ma
numlabel, add

order model model_group brand Fclass Kclass JATOclass

* solve CO2 missing
inspect co2
sum co2
sort year
by year: inspect co2
* the earlier years are most problematic in terms of co2 missings

pwcorr co2 li  hp
* there is very high correlation between co2 and fuel consumption (li)

* simple regression to predict co2 values
reg co2 li
predict pco2
sum pco2

sum co2 pco2 if co2>0
gen diff1=co2-pco2
sum diff1 if co2>0

* some more elaborate regressions
* in levels
reg co2 fuel li we cy wi le he 
predict pco2_ext
sum pco2_ext

sum co2 pco2_ext if co2>0
gen diff2=co2-pco2_ext
sum diff2 if co2>0

* in logs
	gen lco2=log(co2)
	gen lli=log(li)
	gen lwe=log(we)
	gen lcy=log(cy)
	gen lwi=log(wi)
	gen lle=log(le)
	gen lhe=log(he)
	
reg lco2 fuel lli lwe lcy lwi lle lhe 
predict plco2
gen pco2_log=exp(plco2)

sum co2 pco2_log if co2>0
gen diff3=co2-pco2_log
sum diff3 if co2>0

sum pco2_log pco2 pco2_ext co2
sum pco2_log pco2 pco2_ext co2 if co2>0
sum diff1 diff2 diff3 if co2>0

* Findings: diff2 appears to be the smallest among all regressions
* --> use the extended regression in levels to predict the co2 values

* use the predicted values to substitute for the values of missing co2s
replace co2=pco2_ext if co2==.
inspect co2
sum co2

************************************************************************************
*PART 1. DEFINE MARKET DURATION/HOUSEHOLD/NHOME/FCLASS/TREND/OTHER VARIABLES
************************************************************************************
*** MARKET DURATION
gen month 		= month(date)
gen year_intr 	= year(date)

* Permanence in the market;
gen help 	  = 13-month
replace help  = 12 if year>year_intr
replace help  = 0  if year<year_intr
recode help 0 = 1
rename help market_duration
tab market_duration, gen(md)

*** HOUSEHOLD
gen household=population*1000/averagepersonsperhousehold
drop averagepersonsperhousehold

**** FOREIGN
gen nhome=1-home

*** FCLASS

* Drop VANS and MPV's
* Drop SUV and Sports

egen ShareExcluded=total(q) if Fclass==7| Fclass==6 | Fclass==8 | Fclass==9 | JATOclass==12, by(year ma)
bysort year ma (ShareExcluded): replace ShareExcluded=ShareExcluded[1]

drop if Fclass==7| Fclass==6 | Fclass==8 | Fclass==9 | JATOclass==12

*** TREND
* Define the panel dimension of the dataset
egen yema=group(ma year)
tab year, gen (year)


*** OTHER VARIABLES
tab body, gen(body)
tab fuel, gen(fuel)

* generate nhome dummies
gen nhomeFR = nhome*(ma==1)
gen nhomeDE = nhome*(ma==2)
gen nhomeIT = nhome*(ma==3)

************************************************************************************
*PART 2. DELETE LOW SELLING MODELS
************************************************************************************
****** generate q/population *****
gen percentq=(q/population)
sum percentq,d

bys year ma: egen percentqyma=pctile(percentq), p(30)
drop if percentq<percentqyma

************************************************************************************
*PART 3. VARIABLES RELATED TO FUEL COSTS
************************************************************************************

* GASOLINE PRICES: merge. Note that gasoline prices in the UK are already in euro
merge m:1 ma year fuel using gasolinedieselpriceseuroov1000l
drop if _merge==2
drop _merge
drop fuel_string
// gp stands for gasoline price. Data are from excel file provided by Mathias from Datastream
// price is in Euro/1000liters of gasoline or diesel
rename price_1000liter gp
label variable gp "price in Euro/1000liters of gasoline or diesel"
* 1. Generate EUROKM: euro for 1km (on average 7cent per km
gen eurokm = (li/100)*(gp/1000)

*** increasing fuel efficiency over time
gen e=1
table e year, contents(m li)
table e year, contents(m princ)

************************************************************************************************************************
* OTHER VARIABLES
************************************************************************************************************************

* ma = 1 France
* ma = 2 Germany
* ma = 3 Italy

* correct co2 mistake
replace co2 = 140 if model_jato==518 & version_jato==1887 & year==1998

* Correct price for feebate in France and scrapping schemes in France, Italy and Germany 

* Could possibly add the scrapping schemes?

* PRINC INCLUSIVE/EXCLUSIVE TAXES

* Correct for feebate in France

gen feebate = 0

* France

* 2008 and 2009

replace feebate = 1000 if year==2008 & 	ma==1 & co2<100
replace feebate = 700  if year==2008 & 	ma==1 & co2<120
replace feebate = 200  if year==2008 & 	ma==1 & co2<130


replace feebate = -200 		if year==2008 | year==2009 & 	ma==1 & co2>161 & co2<165
replace feebate = -750  	if year==2008 | year==2009 & 	ma==1 & co2>166 & co2<200
replace feebate = -1600  	if year==2008 | year==2009 & 	ma==1 & co2>201 & co2<250
replace feebate = -2600  	if year==2008 | year==2009 & 	ma==1 & co2>251


* 2010
replace feebate = 1000 if year==2010 & 	ma==1 & co2<95
replace feebate = 400  if year==2010 & 	ma==1 & co2<115
replace feebate = 100  if year==2010 & 	ma==1 & co2<125


replace feebate = -200 		if year==2010 & 	ma==1 & co2>156 & co2<160
replace feebate = -750  	if year==2010 & 	ma==1 & co2>161 & co2<195
replace feebate = -1600  	if year==2010 & 	ma==1 & co2>196 & co2<245
replace feebate = -2600  	if year==2010 & 	ma==1 & co2>246

* 2011

replace feebate = 100  if year==2010 & 	ma==1 & co2<105

replace feebate = -200  	if year==2010 & 	ma==1 & co2>150 & co2<155
replace feebate = -750  	if year==2010 & 	ma==1 & co2>156 & co2<190
replace feebate = -1600  	if year==2010 & 	ma==1 & co2>191 & co2<240
replace feebate = -2600  	if year==2010 & 	ma==1 & co2>241

*** Scrapping incentives

gen incentive = 0


* France: Scrapping incentive
replace incentive = 300 	if year==2008 	& ma==1 	& co2<161
replace incentive = 1000 	if year==2009 	& ma==1 	& co2<161
replace incentive = 700 	if year==2010 	& ma==1 	& co2<156
replace incentive = 500 	if year==2010 	& ma==1 	& co2<156
replace incentive = 300 	if year==2011 	& ma==1 	& co2<151

* Germany: scrapping incentive

replace incentive=2500 		if year==2009 & ma==2 

* for Italy (in EUR as given in the respective sources)

replace incentive = 516 	if year==1998 & ma==3 & (li>6 & li<10) /*1998 scheme; depends upon fuel consumption*/
replace incentive = 646 	if year==1998 & ma==3 & li<7

* horsepower is expressed in kw
replace incentive=(150.81+70.00+41.58)+(2.58*hp*3) if year==2002 & ma==3 & hp<86 /*2002 scheme*/
* car with horsepower less than 85 kw can profit from the scheme

replace incentive=800+(2.58*hp*2) if year==2007 & ma==3 & 	hp<101 & co2<141 /*2007 scheme*/
replace incentive=800+(3.87*hp*2) if year==2007 & ma==3 & 	hp>100 & co2<141 /*2007 scheme*/

replace incentive=800 if year==2008 & ma==3 & co2<131 /*2008 scheme*/
replace incentive=900 if year==2008 & ma==3 & co2<121 /*2008 scheme*/

replace incentive=1500 if year==2009 & ma==3 & co2<141 & fuel==1 /*2009 scheme*/ /*for petrol cars*/
replace incentive=1500 if year==2009 & ma==3 & co2<131 & fuel==2 /*2009 scheme*/ /*for diesel cars*/


gen price_net = price_retail_adjusted_Alleuro - feebate - incentive
gen taxprinc =  price_net/gdppercapita

gen taxprinc_old =  price_retail_adjusted_Alleuro/gdppercapita

sum princ taxprinc taxprinc_old

gen size = (wi*le)/1000
sum size
local characteristics "hp li eurokm wi he le size nhome"
sum `characteristics'

*** Rescale to avoid optimization issues

replace hp = hp*0.01
replace li = li*0.1
replace eurokm = eurokm*10
replace wi = wi*0.01
replace he = he*0.01
replace le = le*0.01

label var hp "kW/100"
label var li "liter/100km /10"
label var eurokm "euro/10km"
label var wi "width cm/100"
label var he "height cm/100"
label var le "lenght cm/100"
label var size "wi*le cm2/1,000"

sum hp li wi he le eurokm size

********************************************************************************
* COST INSTRUMENTS
********************************************************************************
****************************************************************************************
* 3. D0 Weight corrections for 2010-2011
***************************************************************************************;
***** In 2010 the definition of weight changes from curbed weight to non curbed weight.
/* By keeping a set of cars that do not change in any characteristic between 2009/2010 
I can regress curbed weight on non curbed weight and then predict weight for the remaining
years.*/
* Identify cars with exactly the same characteristics ;
egen superversion=group(ma brand model_group body Fclass fuel cy hp le wi he li)
/* Get 2009 weight in 2010 if the vehicle does not change and if superversion identifies
weight uniquely*/
gen we2009=we if year==2009
egen sdwe2009=sd(we2009), by(superversion) 
replace sdwe2009=0 if year==2009 & sdwe2009==.
replace we2009=. if sdwe2009>0

egen we2010=min(we2009), by(superversion)  
egen sdwe2010=sd(we2009), by(superversion) 

* Predict Weight given grossweight and cylinder:; 
reg we2010 grosswe cy if year==2010 
predict we_pred if year>2009 
replace we=we_pred if we==. & year>2009 
drop we2009 sdwe2009 we2010 sdwe2010 we_pred superversion

egen totq=sum(q)

*** ppi is by plant of origin
drop if ppi==.
drop if ulc==.
drop if we==.

egen totq2=sum(q)
sum totq totq2
* I lose 2,702,528, which is 3% of the market -> that's acceptable
drop totq totq2

* Hp over Weight
gen hpwe=hp/we
gen volume=le*wi*he
* Make steelcontent and ULC instrument

gen steelcontent=(steelpriceindex*we)/ppi
gen steelvolume=(steelpriceindex*volume)/ppi
replace ulc=ulc/ppi
gen lulc=ln(ulc)

* Dummy for production within sales market
gen localproduct=0
replace localproduct=1 if ma==1 & country1==4
replace localproduct=1 if ma==2 & country1==13
replace localproduct=1 if ma==3 & country1==14
replace localproduct=1 if ma==4 & country1==42
replace localproduct=1 if ma==5 & country1==19
replace localproduct=1 if ma==6 & country1==24
replace localproduct=1 if ma==7 & country1==28
replace localproduct=1 if ma==8 & country1==35

replace localproduct=1 if ma==1 & country2==4
replace localproduct=1 if ma==2 & country2==13
replace localproduct=1 if ma==3 & country2==14
replace localproduct=1 if ma==4 & country2==42
replace localproduct=1 if ma==5 & country2==19
replace localproduct=1 if ma==6 & country2==24
replace localproduct=1 if ma==7 & country2==28
replace localproduct=1 if ma==8 & country2==35

replace localproduct=1 if ma==1 & country3==4
replace localproduct=1 if ma==2 & country3==13
replace localproduct=1 if ma==3 & country3==14
replace localproduct=1 if ma==4 & country3==42
replace localproduct=1 if ma==5 & country3==19
replace localproduct=1 if ma==6 & country3==24
replace localproduct=1 if ma==7 & country3==28
replace localproduct=1 if ma==8 & country3==35

* Rescale IV's (saves code in matlab)
replace ulc=ulc*1000
replace steelcontent=steelcontent*0.001

sum steelcontent lulc localproduct

* Check is there is any residual missing on variables I need
misstable summarize

save model2INST, replace
