************************************************************************************************************************
*** This file creates the instruments for the logit regressions and does the logit
************************************************************************************************************************
clear
clear matrix
capture log close
set more off
set mem 600m
set matsize 800
use model2INST.dta

log using inst, replace
xtset model yema

***********************************************************************************************************************
*Part 1. INSTRUMENTS
***********************************************************************************************************************
local instruments "hp li wi he le eurokm size nhome fuel"

gen potential_mkt = household - ShareExcluded

local potential_mkt "potential_mkt"

sum `characteristics'

gen fuelma1 = fuel2*ma1
gen fuelma2 = fuel2*ma2
gen fuelma3 = fuel2*ma3

***********************************************************************************************************************
* LOGIT INSTRUMENTS

* Sum of the characteristics of all competing products and competing products produced by other firms

* Per year and market;

/* 	i_hp= sum(hp)-hp 					sum of the characteristics of all competing products
	if_hp= sum(hp of firm j) - hp  		sum of the characteristics of all other makes of firm j
	CFi_hp= sum(hp)- sum(hp of firm j)	sum of the characteristics of all other non-firm j makes */

foreach inst in `instruments'{
qui{
	egen i_`inst'=sum(`inst'), by (year ma)
	egen if_`inst'=sum(`inst'), by (year ma firm)
	gen CFi_`inst'=i_`inst'-if_`inst'
	replace i_`inst'=i_`inst'-`inst'
	replace if_`inst'=if_`inst'-`inst'
}
}
qui egen tot_sales 	= sum(q), by(year ma)
qui egen i_num 		= count(q), by(year ma)

qui egen firm_sales = sum(q), by(year ma firm)
qui egen if_num 	= count(q), by(year ma firm)

qui gen CFi_num=i_num-if_num

replace i_num=i_num-1
gen lni_num=ln(i_num)

replace if_num=if_num-1
gen lnif_num=ln(if_num)

***********************************************************************************************************************
* ONE LEVEL NESTED LOGIT INSTRUMENTS, WHERE SEGMENT IS THE PRINCIPLE OF DIFFERENTIATION

* Per year market firm and nest

*** 	is_hp= sum(hp in nest k)-hp 								  	sum of the characteristics of all competing products in nest k
***	    ifs_hp= sum(hp of firm j in nest k) - hp  					sum of the characteristics of all other makes of firm j in nest k
***	    CFis_hp= sum(hp in nest k)- sum(hp of firm j in nest k)	sum of the characteristics of all other non-firm j makes in nest k


foreach inst in `instruments'{
qui{
	egen is_`inst' 		= sum(`inst'),  by (year ma Fclass)
	egen ifs_`inst'	 	= sum(`inst'), by (year ma Fclass firm)
	gen CFis_`inst' 	= is_`inst'-ifs_`inst'
	replace is_`inst' 	= is_`inst'-`inst'
	replace ifs_`inst' 	= ifs_`inst'-`inst'
}
}

egen seg_sales 	= sum(q), by(year ma Fclass)
egen is_num 	= count(q), by(year  ma Fclass)

egen fs_sales 	= sum(q), by(year ma Fclass firm)
egen ifs_num 	= count (q), by(year ma Fclass firm)

qui gen CFis_num = is_num-ifs_num
replace is_num 	= is_num-1
gen lnis_num 	= ln(is_num)
replace ifs_num = ifs_num-1
gen lnifs_num 	= ln(ifs_num)

***********************************************************************************************************************

* CREATION OF THE DEPENDENT VARIABLES

gen lsj=ln(q)-ln(`potential_mkt'-tot_sales) 	
/*dependent, pay attention to the name that is a bit inappropiate: ln(sk)-ln(s0)*/

gen lsjg 	 = ln(q)-ln(seg_sales)

gen lsg 	 = ln(seg_sales)-ln(`potential_mkt')

gen sj  = q/(`potential_mkt')
gen s0 	= (`potential_mkt'-tot_sales)/`potential_mkt'
gen sjg = exp(lsjg)
gen sg  = exp(lsg)

gen sj2=sjg*sg
sum sj sj2
drop sj2

gen lsj2=ln(sj)-ln(s0) 
sum lsj lsj2
drop lsj2


********** FLEXIBLE SPECIFICATION FOR FCLASS

*** Interaction only with F class

tab Fclass, gen(Fclass)

local help "is_hp is_li is_wi is_he is_size is_nhome is_fuel is_num CFis_hp CFis_li CFis_wi CFis_he CFis_size CFis_nhome CFis_fuel CFis_num  ifs_hp ifs_li ifs_wi ifs_he ifs_size ifs_nhome ifs_fuel ifs_num"

foreach x in `help'{
	forvalues y = 1/5{
	gen `x'_Fclass`y'=`x'* Fclass`y'
}
}

*** Generate iteraction between conditional market shares and segment

local help "lsjg"

foreach x in `help'{
forvalues y = 1/5{
qui gen `x'_Fclass`y'=`x'* Fclass`y'
}
}

***********************************************************************************************************************

* ONE LEVEL NESTED LOGIT INSTRUMENTS FOR ORDERED NESTS

* M=2

gen FONclass0 = 1 if Fclass==1
gen FONclass1 = 1 if Fclass==1 | Fclass==2
gen FONclass2 = 1 if Fclass==1 | Fclass==2 | Fclass==3
gen FONclass3 = 1 if Fclass==2 | Fclass==3 | Fclass==4
gen FONclass4 = 1 if Fclass==3 | Fclass==4 | Fclass==5
gen FONclass5 = 1 if Fclass==4 | Fclass==5
gen FONclass6 = 1 if Fclass==5


qui{
foreach inst in `instruments'{
forv num =0/6{
	egen 	FONis_`inst'`num'	=  sum(`inst'),  by (year ma FONclass`num')
	replace FONis_`inst'`num'	= (FONis_`inst'`num' * FONclass`num')-`inst'
	egen 	FONifs_`inst'`num'	= sum(`inst'), by (year ma FONclass`num' firm)
	replace FONifs_`inst'`num'	= (FONifs_`inst'`num' * FONclass`num')-`inst'	
}
}
foreach inst in `instruments'{
	egen FONis_`inst' 		= rowtotal(FONis_`inst'0 FONis_`inst'1 FONis_`inst'2 FONis_`inst'3 FONis_`inst'4 FONis_`inst'5 FONis_`inst'6)
	egen FONifs_`inst' 		= rowtotal(FONifs_`inst'0 FONifs_`inst'1 FONifs_`inst'2 FONifs_`inst'3 FONifs_`inst'4 FONifs_`inst'5 FONifs_`inst'6)

}

forv num =0/6{
egen FONis_num`num'	   = count(q), by(year ma FONclass`num')
replace FONis_num`num' = FONis_num`num'	  -1
replace FONis_num`num' = FONis_num`num' * FONclass`num'

}
egen FONis_num 	= rowtotal(FONis_num0 FONis_num1 FONis_num2 FONis_num3 FONis_num4 FONis_num5 FONis_num6) 

forv num =0/6{
egen FONifs_num`num'	= count(q), by(year ma firm FONclass`num')
replace FONifs_num`num'	= FONifs_num`num'-1
replace FONifs_num`num' = FONifs_num`num' * FONclass`num'
}
}
egen FONifs_num 	= rowtotal(FONifs_num0 FONifs_num1 FONifs_num2 FONifs_num3 FONifs_num4 FONifs_num5 FONifs_num6) 

#delimit ;
mvencode    FONis_hp0 FONis_hp1 FONis_hp2 FONis_hp3 FONis_hp4 FONis_hp5 FONis_hp6   
			FONis_li0 FONis_li1 FONis_li2 FONis_li3 FONis_li4 FONis_li5 FONis_li6  
			FONis_wi0 FONis_wi1 FONis_wi2 FONis_wi3 FONis_wi4 FONis_wi5 FONis_wi6
			FONis_he0 FONis_he1 FONis_he2 FONis_he3 FONis_he4 FONis_he5 FONis_he6
			FONis_nhome0 FONis_nhome1 FONis_nhome2 FONis_nhome3 FONis_nhome4 FONis_nhome5 FONis_nhome6
			FONis_fuel0 FONis_fuel1 FONis_fuel2 FONis_fuel3 FONis_fuel4 FONis_fuel5 FONis_fuel6
			FONis_size0 FONis_size1 FONis_size2 FONis_size3 FONis_size4 FONis_size5 FONis_size6 
			FONis_num0 FONis_num1 FONis_num2 FONis_num3 FONis_num4 FONis_num5 FONis_num6

			FONifs_hp0 FONifs_hp1 FONifs_hp2 FONifs_hp3 FONifs_hp4 FONifs_hp5 FONifs_hp6
			FONifs_li0 FONifs_li1 FONifs_li2 FONifs_li3 FONifs_li4 FONifs_li5 FONifs_li6  
			
			FONifs_wi0 FONifs_wi1 FONifs_wi2 FONifs_wi3 FONifs_wi4 FONifs_wi5 FONifs_wi6 
			FONifs_he0 FONifs_he1 FONifs_he2 FONifs_he3 FONifs_he4 FONifs_he5 FONifs_he6
			FONifs_size0 FONifs_size1 FONifs_size2 FONifs_size3 FONifs_size4 FONifs_size5 FONifs_size6   
			FONifs_nhome0 FONifs_nhome1 FONifs_nhome2 FONifs_nhome3 FONifs_nhome4 FONifs_nhome5 FONifs_nhome6
			FONifs_fuel0 FONifs_fuel1 FONifs_fuel2 FONifs_fuel3 FONifs_fuel4 FONifs_fuel5 FONifs_fuel6
			FONifs_num0 FONifs_num1 FONifs_num2 FONifs_num3 FONifs_num4 FONifs_num5 FONifs_num6
			FONis_hp FONis_li FONis_wi FONis_nhome FONis_fuel FONis_num
			FONifs_hp FONifs_li FONifs_wi FONifs_nhome FONifs_fuel FONifs_num, mv(0) override;
			
#delimit cr		


******************************************************************

*** Gandhi and Houde instrument based on distance

*** sum (x1 - average in a group) = 1/(N-1) sum(xi-xj)

foreach inst in `instruments'{
	forvalues y = 1/5{
		egen 	 sumdistance_`inst'`y' 	= total(`inst' * Fclass`y'), by(year ma Fclass`y')
		mvdecode sumdistance_`inst'`y', mv(0)
		gen  distance_`inst'`y' 	= (`inst' - sumdistance_`inst'`y')/is_num_Fclass`y'
		
		egen sumcharsquares_`inst'`y'  = total((`inst')^2 * Fclass`y'), by(year ma Fclass`y')
		gen distancesq_`inst'`y' 		= (((is_num_Fclass`y'+1)/is_num_Fclass`y') *(`inst')^2) - ((2 * `inst' * sumdistance_`inst'`y')/(is_num_Fclass`y')) + (sumcharsquares_`inst'`y'/(is_num_Fclass`y'))
		mvencode distance_`inst'`y' distancesq_`inst'`y', mv(0) override
		drop sumdistance_`inst'`y' sumcharsquares_`inst'`y'
	}
}


foreach inst in `instruments'{
	forvalues y = 0/6{
		egen 	 ONsumdistance_`inst'`y' 	= total(`inst'* FONclass`y'), by(year ma FONclass`y')
		gen  	 ONdistance_`inst'`y' 		= (`inst' - ONsumdistance_`inst'`y')/FONis_num`y'
		mvencode ONdistance_`inst'`y', mv(0) override
		
		egen ONsumcharsquares_`inst'`y'  = total((`inst')^2), by(year ma FONclass`y')
		
		gen ONdistancesq_`inst'`y' 		= (((FONis_num`y'+1)/FONis_num`y') *(`inst')^2) - ((2 * `inst' * ONsumdistance_`inst'`y')/(FONis_num`y')) + (ONsumcharsquares_`inst'`y'/(FONis_num`y'))
		mvencode ONdistance_`inst'`y' ONdistancesq_`inst'`y', mv(0) override
		drop 	 ONsumdistance_`inst'`y' ONsumcharsquares_`inst'`y'

	}
}

compress
saveold SEGmodel2EST.dta, replace

log close
exit
