function f = jacobNL(theta,delta,Data)

% jacobNL - This function is calculate the derivative of the mean value
% delta wrt the parameter using the implicit function theorem (see Nevo
% http://faculty.wcas.northwestern.edu/~ane686/supplements/Ras_guide_appendix.pdf
% for the Nested Logit
% 
% jacobNL(thetaNL,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters nested logit
%    input2 - Data
%
% Outputs:
%    Derivative of Delta wrt thetaNL
%
% Subfunctions: NLShareCalculation; multinv; multiprod
%

% Author: Laura Grigolon
% August 2012;

% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
Nsigma          = Data.Nsigma;
multidumseg     = Data.multidumseg;

alpha           = theta(1);
Sigseg          = theta(2:Nsigma+1,1);

% Market Shares
delta = reshape(delta,[prods nmkt]);
[~,sij,sijg,numer1,denom1,denom2] = NLShareCalculation(theta,delta,Data);

% Derivative wrt delta

part1          = sum(((1./(1-Sigseg')) .* sij),2);
part1          = (Sigseg./(1-Sigseg))'.*bsxfun(@times,part1,eye(prods));          % diagonal in multiple dimensions

sijtransp      = permute(sij,[2 1 3 4]);
part2          = multiprod(sijg,sijtransp);

part3          = multiprod(sum(sij,2),sum(sijtransp,1));

derShareDeltij = (part1 - part2 - part3);
derShareDelta  = nanmean(derShareDeltij,4);

% Derivative wrt the random coefficients

part1               = Data.PriceInc ./(1-Sigseg);
part2A              = sum((bsxfun(@times,numer1,part1)),1);
part2               = Sigseg * (part2A ./denom1);
part2(isnan(part2)) = 0;

part3A                  = (1-Sigseg) * ((denom1 .^ (-Sigseg)) .* (part2A));
part3A(isnan(part3A))   = 0;
part3B                  = sum(part3A,2);
part3                   = part3B ./ denom2;

derShareRC1    = bsxfun(@minus,part1,part2);
derShareRC2    = sij .* bsxfun(@minus,derShareRC1,part3);
derShareRC     = nanmean(derShareRC2,4);

% Derivative wrt the sigma segment

part1_help     = delta + alpha .* Data.PriceInc;
part1          = (part1_help.*multidumseg)./ ((1-Sigseg)^2);

part2A                  = sum((part1 .* numer1),1);
part2B                  = bsxfun(@rdivide,part2A,denom1);
part2B(isnan(part2B))   = 0;
part2                   = bsxfun(@minus,(- log(denom1)),(Sigseg *part2B));
part2(isinf(part2))     = 0;

part3A                  = bsxfun(@plus,(- log(denom1)),((1-Sigseg) *part2B));
part3B                  = sum(((denom1.^(1-Sigseg)) .* part3A),2);
part3B(isnan(part3B))   = 0;
part3                   = part3B ./ denom2;
part3(isnan(part3))     = 0;

derShareSigseg1 = bsxfun(@minus,part2,part3);
derShareSigseg2 = sij .* bsxfun(@plus,part1,derShareSigseg1);
derShareSigseg  = nanmean(derShareSigseg2,4);


derShareTheta   = [sum(derShareSigseg,2) , sum(derShareRC,2)];
derDeltaTheta   = multiprod((-multinv(derShareDelta)),derShareTheta);
derDeltaTheta   = [reshape(derDeltaTheta(:,1,:,:),[Data.nobs 1]) , reshape(derDeltaTheta(:,2,:,:),[Data.nobs 1])]; % 2D market shares

f = derDeltaTheta;