function [shuffleddraws] = MLHS(D,N)

%-------------------------------------------------------------------------%
% DESCRIPTION:
% This function generates draws following the Modified Latin Hypercube
% Sampling (MLHS) as described in Hess, Train and Polak (2006). The
% function's first argument is the number of dimensions, D. The function's
% second argument is the number of draws per dimension, N. The function 
% returns an N by D matrix where the points in D-dimensional space are 
% stored row-wise.
% This function supplies only the raw draws. Any specific distributional
% assumption in each dimension must be imposed by the user after the draws
% have been generated.
%-------------------------------------------------------------------------%
% REFERENCES:
% Hess, Train and Polak, (2006), ``On the Use of a Modified Latin Hypercube
% Sampling (MLHS) method in the estimation of a Mixed Logit Model for
% vehicle choice," Transportation Research Part B, pp. 147-163.
%-------------------------------------------------------------------------%
% COMPUTATIONAL COMMENT:
% On a first generation MacBook Air, it takes less than 0.4 seconds to 
% generate MLHS draws for 200 dimensions and 10,000 draws in each 
% dimension. It makes therefore no sense to tune this function for faster 
% execution.
%-------------------------------------------------------------------------%

if D < 1
    error('illegal dimension','The number of dimensions must be strictly positive.')
elseif N < 1
    error('illegal number of draws','The number of points in each dimension must be strictly positive.')
else
    ordereddraws = zeros(N,D);
    shuffleddraws = zeros(N,D);
    for i = 1:D
        ordereddraws(:,i) = ((1:N)-1)/N + rand()/N;
        shuffle = randperm(N);
        shuffleddraws(:,i) = ordereddraws(shuffle,i);
    end
end