clear;

% 1. Simulation settings
% Vector of seeds that creates datasets
R           =  500;     % Number of MC simulations
seedvec     =  1:100000/R:100000;

% 1.2. TRUE PARAMETER VALUES

% Size of the datasets
nmkt        = 10;                         % number of markets = (# of cities)*(# of quarters)
prods       = 100;                        % number of brands per market
nobs        = nmkt*prods;                 % number of observations
meanc       = [2 2];                      % mean of the generated vectors:
betatrue    = [-5 ; -1];                  % true mean tastes on constant and X1
Nnest       = 5;                          % number of nests without outside good
% Setting RCL model - true DGP
% Sigma = [sigma11 ; sigma12
%          sigma21 ; sigma22]
sigma       = [6 0.5; 0.5 0.5];             % Random coefficient on the constant, the continous variable, and the interaction - needs to be symmetric

% we will estimate the cholesky elements
% L     = [theta1 ; 0
%          theta2 ; theta3]
Lchol       = chol(sigma,'lower');
chol_true1  = Lchol(1,1);
chol_true2  = Lchol(2,1);
chol_true3  = Lchol(2,2);
chol_true   = [chol_true1;chol_true2;chol_true3];

% Nesting parameter
% Sigma       = zeros(Nnest,1);                         % Sigma has the dimension of the number of nests
Sigma       = 0;
Nsigma      = size(Sigma,1);
Pho         = 0;                         % Pho has dimension number of nests (+OUTSIDE GOOD NEST) + M

% Setting ONGEV model - can be changed
M           = 1;                      % number of neighbors that you have at each side
weight      = [1/2;1/2];          % Weight of each neighboring nest. This has to sum up to 1 and the dimension is M+1 x 1
ndraws      = 100;

% 2. Start Simulation
rng default
ksi     = randn(nobs,1)*0.25;
ksi     = ksi - mean(ksi); % rescale to have E[ksi]=0
const   = ones(nobs,1);

% Number of parameters
nlin    = size(betatrue,1);         % number of linear parameters
nrc     = 2;     % number of random coeficients for Random coefficient Logit

% Individual taste,integration of market shares using Modified Latin Hypercube
draws = cell(nmkt,1);
for i = 1:nmkt
    draws{i} = MLHS(nrc,ndraws);
    draws{i} = norminv(draws{i});
    %     imposing a normal distribution on the draws
end

% Convert to multivariate draws with nonlinear parameter estimation

v=zeros(nmkt,ndraws*nrc);
for nn = 1:nmkt
    vmkt=draws{nn};
    v(nn,:)=reshape(vmkt,ndraws*nrc,1);
end

IDmkt = reshape((repmat((1:1:nmkt)',[1,prods]))',[nmkt*prods 1]);
v=v(IDmkt,:);

clear draws IDmkt

% Store results
% The order of the coefficients is:
%[Constant; Mean value of X1 (a product Attribute); Random Coefficient
Coefftrue            = [betatrue;chol_true];        % true parameters of the model
Coefficients         = NaN(numel(Coefftrue),4,R);   % 4 estimators (Logit, NL, RCLogit, ONGEV)
ComputationTime      = NaN(4,R);

TaboutElast          = NaN(Nnest,Nnest,5,R);

FunctionValue        = NaN(1,4,R);
ExitFlag             = NaN(1,4,R);
Gradient             = NaN(1,4,R);
AIC                  = NaN(1,4,R);
BIC                  = NaN(1,4,R);

%% Set correlation: parameter that determines the extent the segment is informative about the continuous characteristic
corrDstarX1  = 0.9;
stdevDstar   = 1;                             % std. dev. of dstar (does not matter)
stdevX1      = 1;                             % std. dev. of X1

Chelp1       = [stdevDstar^2, corrDstarX1*stdevX1*stdevDstar; corrDstarX1*stdevX1*stdevDstar, stdevX1^2];    %varcovar matrix

for MC = 1:R
    rng(seedvec(MC))
    
    % 3. Simulate data
    disp(['Simulation Numb: ',num2str(MC)])
    
    X1        = zeros(prods,nmkt);
    nesthelp  = zeros(prods,nmkt);
    Dstarhelp = zeros(prods,nmkt);
    
    % triangular distribution
    
    for j = 1:nmkt
        draws    = rand(prods,1);
        X1(:,j)  = (sqrt(2.*draws)-1) .* (draws<=0.5) + (1-sqrt(2.*(1-draws))) .* (draws >0.5);
    end
    X1          = abs((reshape(X1, [nobs 1]))*3);
    nesthelp    = ceil(Nnest *3* tiedrank(X1) / length(X1)); % generate equally size nests
    %      nesthelp    = ceil(7 *4* tiedrank(X1) / length(X1)); % generate equally size nests
    
    % make lower nests more numerous
    nest = NaN(nobs,1);
    nest(nesthelp<=5)              = 1;
    nest(nesthelp>5&nesthelp<=9)   = 2;
    nest(nesthelp>9&nesthelp<=12)  = 3;
    nest(nesthelp>12&nesthelp<=14) = 4;
    nest(nesthelp>14)              = 5;
    
    dumseg        = dummyvar(nest);
    multidumseg   = reshape(dumseg',[Nnest,prods,nmkt]);
    multidumseg   = permute(multidumseg,[2 1 3]);
    
    
    % check if there are empty nests and go to the next iteration if there
    % are (cannot calculate market shares with empty nests)
    test = sum(multidumseg);
    test2 = sum(squeeze(sum(test==0)));
    
    if test2>0
        disp('empty nest - go to next iteration')
        continue
    end
    
    Xexo                = [const X1];
    deltaTrue           = Xexo*betatrue+ksi;
    
    %Step 1: Multiply fixed Standard Normal draws  by lower triangular Cholesky matrix
    % parameters to get the multivariate heterogeneity draws on intercepts and
    % prices
    aw1 = chol_true1 .* v(:,1:ndraws);
    aw2 = chol_true2 .* v(:,1:ndraws) + chol_true3 .* v(:,ndraws+1:ndraws*2);
    
    % Draws*characteristics
    
    xvu1 = v(:,1:ndraws) .* const;
    xvu2 = v(:,1:ndraws) .* X1;
    xvu3 = v(:,ndraws+1:ndraws*2) .* X1;
    
    % Interact with characteristics (constant and price)
    mutrue    = aw1 .* const + aw2 .* X1;
    
    multixvu1 = reshape(xvu1,[prods,1,nmkt,ndraws]);
    multixvu2 = reshape(xvu2,[prods,1,nmkt,ndraws]);
    multixvu3 = reshape(xvu3,[prods,1,nmkt,ndraws]);
    
    multixvu_mu = reshape(mutrue,[prods,1,nmkt,ndraws]);
    
    %% Add outside good as the inferior quality good: get rid of this part if you do not want the outside good
    nestou              = zeros((prods+1),nmkt);
    nestr               = reshape(nest,[prods nmkt]);
    nestou(2:1:end,:)   = nestr;
    nestou              = reshape(nestou,[(prods+1)*nmkt,1]);
    nestou              = nestou+1;     % this is to get positive matrix indices;
    
    %% Weighting matrix
    weight = weight(:,ones(Nnest+1,1));
    % create weighting matrix
    weightmat = zeros(Nnest+M+1,Nnest+1,1);
    for j = 1:(Nnest+1)
        weightmat(j:1:((M+1)+(j-1)),j) = weight(:,j);
    end
    weightmat = weightmat(:, :, ones(nmkt, 1));
    
    % Market share calculation
    
    dumnest       = dummyvar(nestou);
    multidumnest  = reshape(dumnest',[Nnest+1,(prods+1),nmkt]);
    multidumnest  = permute(multidumnest,[2 1 3]);
    multiX1       = reshape(X1,[prods,1,nmkt]);
    
    Data.X1                 = X1;
    Data.nrc                = numel(chol_true);
    Data.xvu1               = xvu1;
    Data.xvu2               = xvu2;
    Data.xvu3               = xvu3;
    Data.ndraws             = ndraws;
    Data.nest               = nestou;
    Data.nmkt               = nmkt;
    Data.prods              = prods;
    Data.nobs               = nobs;
    Data.RandCoeff          = chol_true;
    Data.SigmaNest          = Sigma;
    Data.Pho                = Pho;
    Data.Nnest              = Nnest+1; % include outside good
    Data.M                  = M;
    Data.dumnest            = dumnest;
    Data.multidumnest       = multidumnest;
    Data.multidumseg        = multidumseg;
    Data.weightmat          = weightmat;
    Data.Nsigma             = Nsigma;
    Data.nlin               = nlin;
    Data.multixvu1          = multixvu1;
    Data.multixvu2          = multixvu2;
    Data.multixvu3          = multixvu3;
    Data.deltaTrue          = deltaTrue;
    Data.muTrue             = mutrue;
    
    
    [sj,~]                    = LogitShareCalculation(chol_true,deltaTrue,Data);
    [~,~,sjtrue,sjg,~,~, ~,~,~]    = TrueShare(deltaTrue,Data);
    
    sk                        = squeeze(sj);
    lnsj                      = log(sk);
    Data.logobsshare          = lnsj;
    
    % 2D reshape
    s0      = bsxfun(@minus,ones(prods,1),sum(sj,1));
    % 2D reshape
    sj                         = reshape(sum(sj,2),[prods*nmkt 1]);
    s0                         = reshape(s0,[prods*nmkt 1]);
    y                          = log(sj) - log(s0);             % log-odds ratio
    sjg                        = reshape(sum(sjg,2),[prods*nmkt 1]);
    lnsjg                      = log(sjg);
    
    %% 5 Optimal instruments
    
    % Optimal instruments RCL
    
    % Delta without ksi
    theta0            = chol_true;
    derShareDelta     = zeros(prods,prods,nmkt);
    derShareTheta     = zeros(prods,numel(theta0),nmkt);
    
    PseudoDelta       = reshape(Xexo*betatrue,[prods 1 nmkt]);
    
    for i=1:nmkt
        
        Data.xvu1MA           = xvu1(prods*(i-1)+1:prods*i,:);
        Data.xvu2MA           = xvu2(prods*(i-1)+1:prods*i,:);
        Data.xvu3MA           = xvu3(prods*(i-1)+1:prods*i,:);
        
        PseudoDeltaMA         = PseudoDelta(:,:,i);
        multidumnestMA        = multidumnest(:,:,i);
        weightmatMA           = weightmat(:,:,i);
        Data.multidumnestMA   = multidumnestMA;
        Data.weightmatMA      = weightmatMA;
        Data.PseudoDeltaMA    = PseudoDeltaMA;
        fHandle               = @RcMA;
        derShareDelta(:,:,i)  = numgrad(fHandle,PseudoDeltaMA,Data);
        fHandle               = @RcMATheta;
        derShareTheta(:,:,i)  = numgrad(fHandle,theta0,Data);
    end
    
    derDeltaTheta   = multiprod((-multinv(derShareDelta)),derShareTheta);
    derDeltaTheta   = permute(derDeltaTheta,[2 1 3]);
    chamberlin      = reshape(derDeltaTheta, [numel(theta0) nobs]);
    chamberlin      = chamberlin';
    
    Z               = [Xexo chamberlin];
    
    Data.alpha      = betatrue(nlin);
    deltaEX         = deltaTrue - Data.alpha*X1;
    Data.deltaEXMA  = deltaEX(1:prods,:);
    
    %% True Elasticity
    PriceOverShare  =  (1./sj(1:prods)) * (X1(1:prods,1))';
    
    derSharePrice  = numgrad(@LogitSharePriceRCMA,X1(1:prods),Data);
    ElaTRUE        = bsxfun(@times,derSharePrice,PriceOverShare);
    
    % Segment level elasticity
    %% True Elasticity
    taboutTrueElast = zeros(Nnest,Nnest);
    DiversionTrue   = zeros(Nnest,Nnest);
    
    % Cross price elasticity neighboring same seg: mkt share after price increase 1%
    for j = 1:Nnest
        % price increase per segment in market 1
        lambda          = 0.01;
        selX1           = bsxfun(@times,X1(1:prods,:),dumseg(1:prods,:)); %PRODS X NEST
        newX1           = selX1(:,j) .* (1+lambda);
        selX1(:,j)      = newX1;
        selX1           = sum(selX1,2);
        [skNEW,~]       = LogitSharePriceRCMA(selX1,Data);
        skNEW           = sum(skNEW,2);
        demand0Nest     = sum(sj(1:prods,1).*(j==nest(1:prods)));
        demand1Nest     = sum(skNEW(1:prods,1).*(j==nest(1:prods)));
        
        for jj = 1:Nnest
            demand0                = sum(sj(1:prods,1).*(jj==nest(1:prods)));      % Demand before price increase
            demand1                = sum(skNEW(1:prods,1).*(jj==nest(1:prods)));   % Demand after price increase
            taboutTrueElast(j,jj)  = 100*((demand1-demand0)./ demand0);
        end
    end
    
    TaboutElast(:,:,5,MC)         = taboutTrueElast;
    
    %% 6 Estimation
    
    %% Logit
    % Weighting Matrix
    W           = (Z'*Z)\eye(size(Z,2));
    BLog        = ((Xexo'*Z*W*Z'*Xexo)\eye(size(Xexo,2)))*(Xexo'*Z*W*Z'*y);
    est         = y-Xexo*BLog;
    
    deltL       = y;
    alpha       = BLog(nlin);
    
    % Delta exogenous wrt X1
    Data.alpha      = alpha;
    
    deltaEX         = deltL - Data.alpha*X1;
    deltaEXMA       = deltaEX(1:prods,:);
    
    % Logit elasticity wrt X1
    Data.deltaEXMA  = deltaEXMA;
    Data.RandCoeff  = [0;0;0];
    
    derSharePrice  = numgrad(@LogitSharePriceRCMA,X1(1:prods),Data);
    ElaLogit       = bsxfun(@times,derSharePrice,PriceOverShare);
    
    % Demand before price increase
    taboutElastLog = zeros(Nnest,Nnest);
    
    % Cross price elasticity neighboring same seg: mkt share after price increase 1%
    for j = 1:Nnest
        % price increase per segment in all markets
        lambda          = 0.01;
        selX1           = bsxfun(@times,X1(1:prods,:),dumseg(1:prods,:)); %PRODS X NEST
        newX1           = selX1(:,j) .* (1+lambda);
        selX1(:,j)      = newX1;
        selX1           = sum(selX1,2);
        
        [skNEW,~]       = LogitSharePriceRCMA(selX1,Data);
        skNEW           = squeeze(sum(skNEW,2));
        
        demand0Nest          = sum(sj(1:prods,1).*(j==nest(1:prods)));
        demand1Nest          = sum(skNEW(1:prods,1).*(j==nest(1:prods)));
        
        for jj = 1:Nnest
            demand0              = sum(sj(1:prods,1).*(jj==nest(1:prods)));      % Demand before price increase
            demand1              = sum(skNEW(1:prods,1).*(jj==nest(1:prods)));   % Demand after price increase
            taboutElastLog(j,jj) = 100*((demand1-demand0)./ demand0);
            
        end
    end
    
    % Function value
    FctValLog   = (est' * Z) * W * (Z' * est);
    
    % Store the results
    Coefficients(:,1,MC)          = [BLog;NaN(numel(chol_true),1)];
    ComputationTime(1,MC)         = nan;
    
    TaboutElast(:,:,1,MC)         = taboutElastLog;
    
    FunctionValue(:,1,MC)         = FctValLog;
    ExitFlag(:,1,MC)              = nan;
    
    
    %% RC Logit
    
    Z               = [Xexo chamberlin];
    W               = (Z'*Z)\eye(size(Z,2));
    Data.Xexo       = Xexo;
    
    xzwz            = Xexo'*Z*W*Z';
    Data.xzwz       = xzwz;
    xzwzx           = xzwz * Xexo;
    invxzwzx        = xzwzx\eye(size(xzwzx,2));
    Data.invxzwzx   = inv(xzwzx);
    Data.Z          = Z;
    Data.W          = W;
    
    %     OPTIMIZATION
    
    theta20RCL = chol_true ;     % starting value
    
    delta0   = reshape(deltaTrue,prods,nmkt);
    save mvalold delta0
    
    angmm    = @(theta20RCL)gmmLogit(theta20RCL,Data);
    options  = optimset( 'Algorithm','sqp',...
        'GradObj','on','TolCon',1E-6,...
        'TolFun',1E-6,'TolX',1E-6,...
        'Hessian', 'off','DerivativeCheck','off','FinDiffType','central');
    
    t1      = cputime;
    
    [thetaRCL, FctValRCL, exitflagRCL,outputRCL] = fminunc(angmm,theta20RCL, options);
    
    
    % Ricalculate beta according to the thetaRCL that minimizes the function
    % given the use of multistart by KNITRO
    
    deltRCL      = deltaLogit(thetaRCL,Data);
    deltRCL      = reshape(deltRCL,[Data.nobs, 1]);
    beta         = invxzwzx*(xzwz*deltRCL);
    th12RCL      = [beta ; thetaRCL];
    
    
    % retrieve Sigma by transforming back from cholesky
    
    chol_est  = [thetaRCL(1),0;thetaRCL(2),thetaRCL(3)];
    sigma_est = chol_est*chol_est';
    
    th12RCL_est = [beta ; sigma_est(1);sigma_est(4);sigma_est(2)];
    
    cputimegmm   = cputime-t1;
    
    % Random Coefficient Logit elasticity wrt X1inc
    
    % Delta exogenous wrt X1
    Data.alpha      = beta(nlin);
    deltaEX         = deltRCL - Data.alpha*X1;
    deltaEXMA       = deltaEX(1:prods,:);
    Data.RandCoeff  = thetaRCL;
    Data.deltaEXMA  = deltaEXMA;
    derSharePrice   = numgrad(@LogitSharePriceRCMA,X1(1:prods),Data);
    ElaRCL          = bsxfun(@times,derSharePrice,PriceOverShare);
    
    % Demand before price increase
    taboutElastRCL = zeros(Nnest,Nnest);
    DiversionRCL   = zeros(Nnest,Nnest);
    % Cross price elasticity neighboring same seg: mkt share after price increase 1%
    for j = 1:Nnest
        % price increase per segment in all markets
        lambda          = 0.01;
        selX1           = bsxfun(@times,X1(1:prods,:),dumseg(1:prods,:)); %PRODS X NEST X MKT
        newX1           = selX1(:,j) .* (1+lambda);
        selX1(:,j)      = newX1;
        selX1           = sum(selX1,2);
        
        [skNEW,~]       = LogitSharePriceRCMA(selX1,Data);
        skNEW           = squeeze(sum(skNEW,2));
        
        for jj = 1:Nnest
            demand0              = sum(sk(1:prods,1).*(jj==nest(1:prods)));      % Demand before price increase
            demand1              = sum(skNEW(1:prods,1).*(jj==nest(1:prods)));   % Demand after price increase
            taboutElastRCL(j,jj) = 100*((demand1-demand0)./ demand0);
        end
    end
    
    clear demand0 demand1
    
    % Store the results
    Coefficients(:,3,MC)        = [th12RCL(1:nlin);abs(thetaRCL)];
    ComputationTime(3,MC)       = cputimegmm;
    
    TaboutElast(:,:,3,MC)         = taboutElastRCL;
    
    FunctionValue(:,3,MC)         = FctValRCL;
    ExitFlag(:,3,MC)              = exitflagRCL;
    Gradient(:,3,MC)              = outputRCL.firstorderopt;
    
    %% Nested Logit
    
    XNestedLog              = [Xexo lnsjg];
    BNLog                   = ((XNestedLog'*Z*W*Z'*XNestedLog)\eye(size(XNestedLog,2)))*(XNestedLog'*Z*W*Z'*y);
    
    if BNLog(nlin+1)> 0.975 || BNLog(nlin+1)< 0
        disp('Sigma Nested Logit Above 1 or below 0')
        continue
    end
    
    % Standard errors
    est         = y-XNestedLog*BNLog;
    
    deltNL     = y - BNLog(nlin+1)*lnsjg;
    % Nested Logit elasticity wrt X1
    Data.Nsigma = 1;
    alpha      = BNLog(nlin);
    Data.alpha = alpha;
    theta      = [BNLog(nlin+1);0;0];
    
    % Delta exogenous wrt X1
    Data.deltNL     = deltNL;
    deltaEX         = deltNL - Data.alpha*X1;
    Data.deltaEXMA  = deltaEX(1:prods,:);
    Data.Sigseg     = BNLog(nlin+1);
    
    % Elasticity
    derSharePrice  = numgrad(@NLSharePriceMA,X1(1:prods),Data);
    ElaNL          = bsxfun(@times,derSharePrice,PriceOverShare);
    
    % Demand before price increase
    taboutElastNL   = zeros(Nnest,Nnest);
    
    % Cross price elasticity neighboring same seg: mkt share after price increase 1%
    for j = 1:Nnest
        % price increase per segment in all markets
        lambda          = 0.01;
        selX1           = bsxfun(@times,multiX1(:,:,1),multidumseg(:,:,1)); %PRODS X NEST X MKT
        newX1           = selX1(:,j) .* (1+lambda); % just market 1
        selX1(:,j)      = newX1;
        selX1           = squeeze(sum(selX1,2));
        
        skNEW           = NLSharePriceMA(selX1,Data);
        
        for jj = 1:Nnest
            demand0             = sum(sj(1:prods,1).*(jj==nest(1:prods)));      % Demand before price increase
            demand1             = sum(skNEW(1:prods,1).*(jj==nest(1:prods)));   % Demand after price increase
            taboutElastNL(j,jj)   = 100*((demand1-demand0)./ demand0);
            
        end
    end
    
    % Function value
    FctValNL   = (est' * Z) * W * (Z' * est);
    
    % Store the results
    Coefficients(:,2,MC)          = [BNLog;nan;nan];
    ComputationTime(2,MC)         = nan;
    TaboutElast(:,:,2,MC)         = taboutElastNL;
    FunctionValue(:,2,MC)         = FctValNL;
    ExitFlag(:,2,MC)              = nan;
    
    
    %% ONGEV
    theta0            = [0.6;0.2];
    deltaTrueT  = reshape(deltaTrue, [prods nmkt]);
    delta0      = deltaTrueT;
    save mvalold delta0
    
    angmmONGEV  = @(theta0)gmmONGEV(theta0,Data);
    
    options = optimset('Algorithm','sqp',...
        'GradObj','off','TolCon',1E-6,...
        'TolFun',1E-6,'TolX',1E-6,...
        'Hessian', 'off','DerivativeCheck','off','FinDiffType','central');
    
    %     A = [-1,1];
    %     b = 0;
    
    x_L     = zeros(size(theta0));       % lower bound (for nesting parameters to avoid numerical problems when sigma/pho gets negative)
    x_U     = ones(size(theta0))*0.975;   % upper bound (for nesting parameters to avoid numerical problems when sigma/pho gets close to 1)
    
    [thetaONGEV, FctValONGEV, exitflagONGEV,outputONGEV] = fmincon(angmmONGEV,theta0,[],[],[],[], x_L, x_U, [], options);
    
    
    deltONGEV       = delta(thetaONGEV,Data);
    Data.deltONGEV  = deltONGEV;
    deltONGEV       = reshape(deltONGEV,[nobs 1]);
    betaONGEV       = invxzwzx*(xzwz*deltONGEV);
    th12ONGEV       = [betaONGEV ; thetaONGEV];
    cputimegmm      = cputime-t1;
    
    alpha      = th12ONGEV(nlin);
    
    % Delta exogenous wrt X1
    deltaEX     = deltONGEV - Data.alpha*X1;
    deltaEX     = reshape(deltaEX,prods,nmkt);
    deltaEXMA   = deltaEX(1:prods,1);
    
    Data.deltaEXMA  = deltaEXMA;
    Data.theta      = thetaONGEV;
    
    %% Product level elasticity
    
    fHandle         = @ONGEVSharePriceMA;
    derSharePrice   = numgrad(fHandle,X1(1:prods),Data);
    ElaONGEV        = bsxfun(@times,derSharePrice,PriceOverShare);
    
    % Delta exogenous wrt X1
    deltaEX     = deltONGEV - Data.alpha*X1;
    deltaEX     = reshape(deltaEX,prods,nmkt);
    
    Data.deltaEX    = deltaEX;
    Data.theta      = thetaONGEV;
    
    % Demand before price increase
    taboutElastONGEV = NaN(Nnest,Nnest+1);
    % Cross price elasticity neighboring same seg: mkt share after price increase 1%
    for j = 1:Nnest
        % price increase per segment in all markets
        lambda          = 0.01;
        selX1           = bsxfun(@times,X1(1:prods,:),dumseg(1:prods,:)); %PRODS X NEST
        newX1           = selX1(:,j) .* (1+lambda);
        selX1(:,j)      = newX1;
        selX1           = sum(selX1,2);
        
        [skNEW,~]       = ONGEVSharePriceMA(selX1,Data);
        demand0Nest     = sum(sj(1:prods,1).*(j==nest(1:prods)));
        demand1Nest     = sum(skNEW(1:prods,1).*(j==nest(1:prods)));
        
        Outside0                = 1-sum(sj(1:prods,1));
        Outside1                = 1-sum(skNEW(1:prods,1));
        taboutElastONGEV(j,1)    = 100*((Outside1-Outside0)./ Outside0);
        
        for jj = 1:Nnest
            demand0                    = sum(sj(1:prods,1).*(jj==nest(1:prods)));      % Demand before price increase
            demand1                    = sum(skNEW(1:prods,1).*(jj==nest(1:prods)));   % Demand after price increase
            taboutElastONGEV(j,jj+1)   = 100*((demand1-demand0)./ demand0);
        end
    end
    clear demand0Nest demand1Nest demand0 demand1
    
    % AIC/BIC
    
    % Store the results
    Coefficients(:,4,MC)          = [betaONGEV ;thetaONGEV;NaN];
    ComputationTime(4,MC)         = cputimegmm;
    
    TaboutElast(:,:,4,MC)         = taboutElastONGEV(:,2:end);
    
    FunctionValue(:,4,MC)         = FctValONGEV;
    ExitFlag(:,4,MC)              = exitflagONGEV;
    Gradient(:,4,MC)              = outputONGEV.firstorderopt;
    
    
    savefile = 'RCL_ONGEV_B8B9.mat';
    save    (savefile, 'Coefftrue', 'Coefficients', 'ComputationTime',...
        'TaboutElast',...
        'FunctionValue', 'ExitFlag','Gradient')
    
    
end

% Drop draws for which ONGEV did not converge correctly

TaboutElast(:,:,:,Coefficients(3,4,:)==0.975) = []; % sigma close to 1: numerical unstable when denom is close to zero
ComputationTime(:,Coefficients(3,4,:)==0.975) = [];
Coefficients(:,:,Coefficients(3,4,:)==0.975) = [];

TaboutElast(:,:,:,Coefficients(4,4,:)>Coefficients(3,4,:)) = []; % pho>sigma: not consistent with random utility max
ComputationTime(:,Coefficients(4,4,:)>Coefficients(3,4,:)) = [];
Coefficients(:,:,Coefficients(4,4,:)>Coefficients(3,4,:)) = [];

TaboutElast(:,:,:,~isfinite(Coefficients(1,4,:)))  = [];
ComputationTime(:,~isfinite(Coefficients(1,4,:)))  = [];
Coefficients(:,:,~isfinite(Coefficients(1,4,:)))   = [];

% Analysis results
MeanCoefficient         = nanmean(Coefficients,3)
EmpiricalStErrors       = nanstd(Coefficients,0,3)
CompTime                = nansum(ComputationTime,2)
GroupElast              = nanmean(TaboutElast,4)


labels      = {'const'  'xj'  'L11' 'L21' 'L22' 'sigma' 'pho'};
title1      = {'Coeff True', 'Logit','Nested Log','RCL', 'Ordered Nested Log'};

RCLCoeff    = [MeanCoefficient(:,3);NaN;NaN];
NLCoeff     = [MeanCoefficient(1,2);MeanCoefficient(2,2);NaN;NaN;NaN;MeanCoefficient(3,2);NaN];
ONLCoeff    = [MeanCoefficient(1,4);MeanCoefficient(2,4);NaN;NaN;NaN;MeanCoefficient(3,4);MeanCoefficient(4,4)];
Coeff       = [[Coefftrue;NaN;NaN] [MeanCoefficient(:,1);NaN(2,1)] NLCoeff RCLCoeff ONLCoeff];

xlswrite('RCLvsONL_M1',title1, 'Coefficients','B1')
xlswrite('RCLvsONL_M1',labels', 'Coefficients','A2')
xlswrite('RCLvsONL_M1',Coeff, 'Coefficients','B2')

RCLStdDev    = [EmpiricalStErrors(:,3);NaN;NaN];
NLStdDev     = [EmpiricalStErrors(1,2);EmpiricalStErrors(2,2);NaN;NaN;NaN;EmpiricalStErrors(3,2);NaN];
ONLStdDev    = [EmpiricalStErrors(1,4);EmpiricalStErrors(2,4);NaN;NaN;NaN;EmpiricalStErrors(3,4);EmpiricalStErrors(4,4)];
StdDev      = [NaN(7,1) [EmpiricalStErrors(:,1);NaN(2,1)] NLStdDev RCLStdDev ONLStdDev];


xlswrite('RCLvsONL_M1',title1, 'StdDev','B1')
xlswrite('RCLvsONL_M1',labels', 'StdDev','A2')
xlswrite('RCLvsONL_M1',StdDev, 'StdDev','B2')

xlswrite('RCLvsONL_M1',title1(2), 'GroupElasticities','A1')
xlswrite('RCLvsONL_M1',GroupElast(:,:,1), 'GroupElasticities','A2')

xlswrite('RCLvsONL_M1',title1(3), 'GroupElasticities','A7')
xlswrite('RCLvsONL_M1',GroupElast(:,:,2), 'GroupElasticities','A8')

xlswrite('RCLvsONL_M1',title1(4), 'GroupElasticities','A13')
xlswrite('RCLvsONL_M1',GroupElast(:,:,3), 'GroupElasticities','A14')

xlswrite('RCLvsONL_M1',title1(5), 'GroupElasticities','A19')
xlswrite('RCLvsONL_M1',GroupElast(:,:,4), 'GroupElasticities','A20')

xlswrite('RCLvsONL_M1',title1(1), 'GroupElasticities','A25')
xlswrite('RCLvsONL_M1',GroupElast(:,:,5), 'GroupElasticities','A26')