function [sij, sijg, sj, sjg, s0, numer1, denom1, numer2, denom2]=TrueShare(delta,Data)

% TrueShare - Nested Logit share calculation using true parameter values
% Syntax:   [sij sijg sj sjg s0 numer1 denom1 numer2 denom2]=TrueShare(delta,Data)
%
% Inputs:
%    input1 - Mean value delta
%    input2 - Data
%
% Outputs:
%    sij    = individual market shares
%    sijg   = individual conditional share of choosing good j from group g
%    sj     = market shares
%    sjg    = conditional share of choosing good j from group g 
%    s0     = outside good
%    numer1 = numerator1
%    denom1 = denominator1
%    numer2 = numerator 2
%    denom2 = denominator 2
%
% Subfunctions: none
%

% Author: Laura Grigolon and Frank Verboven
% August 2012;

% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
SigsegTrue      = Data.SigmaNest;
ndraws          = Data.ndraws;
muTrue          = Data.muTrue;
multidumseg     = Data.multidumseg;

% Market share calculation
mudel     = bsxfun(@plus,delta,muTrue);
numer1    = exp(mudel);

numer1      = reshape(numer1,[prods,1,nmkt,ndraws]);
numer1      = bsxfun(@times,numer1,multidumseg);

numer1    = bsxfun(@power,numer1,(1./(1-SigsegTrue')));

% use multidimensional arrays
denom1      = nansum(numer1,1);
numer2      = denom1.^(1-SigsegTrue');
denom2      = 1 + sum(numer2,2);                         % sum across segments within a market (second dimension)

sijg              = bsxfun(@rdivide,numer1,denom1);
sijg(isnan(sijg)) = 0;

sig               = bsxfun(@rdivide,numer2,denom2);
sig(isnan(sig))   = 0;

sij               = bsxfun(@times,sijg,sig);
sj                = mean(sij,4);
sg                = mean(sig,4);
sjg               = bsxfun(@rdivide,sj,sg);

sij               = sum(sij,2);
sij(isnan(sij))   = 0;

sj                = sum(sj,2);

s0                = 1 ./ denom2;
s0                = mean(s0(:,:,:,1:ndraws),4);

end